require 'pathname'
require 'jekyll/tagging'

module Jekyll
    class MultiLangTagger < Tagger

        safe true

        @types = [:page, :feed]
 
        def generate(site)
            # Iterate over all posts and group the tags by language.
            for post in site.posts.docs do
                lang = post.data['lang']
                site.config['t'][lang]['tagnames'] = 
                        {} unless site.config['t'][lang]['tagnames']
                tagnames = site.config['t'][lang]['tagnames']
                tags = post.data['tags']
                for tag in tags do
                    slug = jekyll_tagging_slug(tag)
                    if tagnames[slug]
                        if tagnames[slug] && tagnames[slug] != tag
                            raise "Tag '%{tag1}' and tag '%{tag2}' will create the same filename.  Change one of them!" % { :tag1 => tagnames[slug], :tag2 => tag }
                        end
                    else 
                        tagnames[slug] =  tag
                    end
                end
            end

            # Now call the jekyll-tagging plug-in for each language,
            # modifying the site configuration before each invocation.
            saved_tag_page_dir = site.config['tag_page_dir']
            saved_tag_feed_dir = site.config['tag_feed_dir']

            for lang in site.config['t'].keys
                site.config['tag_page_dir'] = '/' + lang + '/' + saved_tag_page_dir
                site.config['tag_feed_dir'] = '/' + lang + '/' + saved_tag_feed_dir
                site.config['ignored_tags'] = site.tags.keys - site.config['t'][lang]['tagnames'].values

                super
            end

            # Now inject the language and the description into the newly 
            # created pages.
            for page in site.pages
                if page.data['tag']
                    dir = Pathname(page.url).each_filename.to_a
                    lang = page.data['lang'] = dir[0]
                    description = site.config['t'][lang]['taglist']['description']
                    page.data['description'] = description % { :tag => page.data['tag'] }
                end
            end

            # Restore configuration.
            site.config['tag_page_dir'] = saved_tag_page_dir
            site.config['tag_feed_dir'] = saved_tag_feed_dir

            # Make the original tagger ignore all tags so that it will not
            # generate any file.
            site.config['ignored_tags'] = site.tags.keys 

        end
    end
end
